<%def capitalizedPluginId = pluginId.capitalize().replaceAll("\\s", "")
%>package ${projectRootPackage}.bpg;

import com.haulmont.studio.common.model.base.*;
import com.haulmont.studio.common.model.Project;

import java.util.List;

public class ${capitalizedPluginId}BaseProject extends BaseProject {
    public ${capitalizedPluginId}BaseProject(List<BaseProject> baseProjects) {
        for (BaseProject baseProject : baseProjects) {
            dependsOn.add(baseProject);
        }

        name = "${projectName}";
        artifactGroup = "${projectRootPackage}";
        required = true;

        coreAppProperties.put("cuba.springContextConfig", "${coreAppProperties['cuba.springContextConfig']}");
        coreAppProperties.put("cuba.persistenceConfig", "${coreAppProperties['cuba.persistenceConfig']}");
        coreAppProperties.put("cuba.metadataConfig", "${coreAppProperties['cuba.metadataConfig']}");
        coreAppProperties.put("cuba.viewsConfig", "${coreAppProperties['cuba.viewsConfig']}");

        webAppProperties.put("cuba.springContextConfig", "${webAppProperties['cuba.springContextConfig']}");
        webAppProperties.put("cuba.dispatcherSpringContextConfig", "${webAppProperties['cuba.dispatcherSpringContextConfig']}");
        webAppProperties.put("cuba.persistenceConfig", "${webAppProperties['cuba.persistenceConfig']}");
        webAppProperties.put("cuba.metadataConfig", "${webAppProperties['cuba.metadataConfig']}");
        webAppProperties.put("cuba.viewsConfig", "${webAppProperties['cuba.viewsConfig']}");
        webAppProperties.put("cuba.windowConfig", "${webAppProperties['cuba.windowConfig']}");
        webAppProperties.put("cuba.menuConfig", "${webAppProperties['cuba.menuConfig']}");
        webAppProperties.put("cuba.permissionConfig", "${webAppProperties['cuba.permissionConfig']}");
        webAppProperties.put("cuba.mainMessagePack", "${webAppProperties['cuba.mainMessagePack']}");
        webAppProperties.put("cuba.creditsConfig", "${webAppProperties['cuba.creditsConfig']}");

        portalAppProperties.put("cuba.metadataConfig", "metadata.xml");

        modules.add(new Global(this));
        modules.add(new Core(this));
        modules.add(new Gui(this));
        modules.add(new Web(this));
        <%if (projectIsPortal) {%>
        modules.add(new Portal(this));
        <%}%>
        <%if (projectIsDesktopClient) {%>
        modules.add(new Desktop(this));
        <%}%>
        <%if (projectIsWebToolkit) {%>
        modules.add(new WebToolkit(this));
        <%}%>
    }

    public static class Global extends BaseModule.Global {

        public Global(BaseProject baseProject) {
            super(baseProject, "${projectModulePrefix}-global");
        }

        @Override
        public String getGAV() {
            return baseProject.getArtifactGroup() + ":" + artifactName + ":\$baseProjectVersion";
        }
    }

    public static class Core extends BaseModule.Core {

        public Core(BaseProject baseProject) {
            super(baseProject, "${projectModulePrefix}-core");
        }

        @Override
        public List<String> getDependencies() {
            List<String> strings = super.getDependencies();
            strings.add("dbscripts(\"" + getGAV() + ":db@zip\")");
            return strings;
        }

        @Override
        public String getGAV() {
            return baseProject.getArtifactGroup() + ":" + artifactName + ":\$baseProjectVersion";
        }
    }

    public static class Gui extends BaseModule.Gui {

        public Gui(BaseProject baseProject) {
            super(baseProject, "${projectModulePrefix}-gui");
        }

        @Override
        public String getGAV() {
            return baseProject.getArtifactGroup() + ":" + artifactName + ":\$baseProjectVersion";
        }
    }

    public static class Web extends BaseModule.Web {

        public Web(BaseProject baseProject) {
            super(baseProject, "${projectModulePrefix}-web");
        }

        @Override
        public List<String> getDependencies(Project project) {
            List<String> strings = super.getDependencies();
            strings.add("webcontent(\"com.haulmont.cuba:cuba-web:\$cubaVersion:web@zip\")");
            if (!project.isWebToolkit()) {<%if (!projectIsWebToolkit) {%>
                strings.add("webcontent(\"com.haulmont.cuba:${webToolkitSuffix}-web-toolkit:\$cubaVersion:web@zip\")");<%} else {%>
                strings.add("webcontent(\"" + baseProject.getArtifactGroup() + ":${projectModulePrefix}-web-toolkit:\$baseProjectVersion:web@zip\")");<%}%>
            }<%if (baseProjectsNames.contains('charts')) {%>
            strings.add("webcontent(\"com.haulmont.charts:charts-web:\$cubaVersion:web@zip\")");
            <%}%>
            return strings;
        }

        @Override
        public String getGAV() {
            return baseProject.getArtifactGroup() + ":" + artifactName + ":\$baseProjectVersion";
        }
    }

<%if (projectIsDesktopClient) {%>
    public static class Desktop extends BaseModule.Desktop {

        public Desktop(BaseProject baseProject) {
            super(baseProject, "${projectModulePrefix}-desktop");
        }

        @Override
        public String getGAV() {
            return baseProject.getArtifactGroup() + ":" + artifactName + ":\$baseProjectVersion";
        }
    }
<%}%>

<%if (projectIsPortal) {%>
    public static class Portal extends BaseModule.Portal {

        public Portal(BaseProject baseProject) {
            super(baseProject, "${projectModulePrefix}-portal");
        }

        @Override
        public String getGAV() {
            return baseProject.getArtifactGroup() + ":" + artifactName + ":\$baseProjectVersion";
        }

        @Override
        public List<String> getDependencies() {
            List<String> strings = super.getDependencies();
            strings.add("webcontent(\"" + getGAV() + ":web@zip\")");
            return strings;
        }
    }
<%}%>

<%if (projectIsWebToolkit) {%>
    public static class WebToolkit extends BaseModule.WebToolkit {

        public WebToolkit(BaseProject baseProject) {
            super(baseProject, "${projectModulePrefix}-web-toolkit");
        }

        @Override
        public String getGAV() {
            return baseProject.getArtifactGroup() + ":" + artifactName + ":\$baseProjectVersion";
        }
    }
<%}%>
}
