def fields = []

view.orderedRootProperties.each { prop ->

    def attribute = entity.getAttribute(prop.name)
    def type = getFieldType(attribute)

    if (type == null) {
        return
    }

    def fieldBinding = [
            'type'         : type,
            'bindPath'     : 'entity.' + attribute.name,
            'errorBindPath': 'serverErrors.' + attribute.name,
            'label'        : "[[msg('$entity.name.$attribute.name')]]",
            'required'     : attribute.mandatory,
            'maxLength'    : attribute.maxLength
    ]

    if (type == 'enum') {
        fieldBinding['enumItems'] = "[[enumValues('${attribute.typeFqn}')]]"
    }

    if (type == 'lookup') {
        fieldBinding['lookupEntityName'] = attribute.entityType.name
        fieldBinding['itemsBindPath'] = "entity_${attribute.name}Options"
    }

    fields << fieldBinding
}


def getFieldType(attribute) {
    if (attribute.mappingType == 'ENUM') {
        return 'enum'
    }
    if (isFile(attribute)) {
        return 'file'
    }
    if (isToOneRelation(attribute)) {
        return 'lookup'
    }
    if (attribute.mappingType == 'DATATYPE') {
        if (attribute.isDeepInheritor('java.lang.Boolean')) {
            return 'boolean'
        }
        if (attribute.isDeepInheritor('java.lang.Integer') || attribute.isDeepInheritor('java.lang.Long')) {
            return 'number'
        }
        if (attribute.isDeepInheritor('java.lang.BigDecimal') || attribute.isDeepInheritor('java.lang.Double')) {
            return 'fpNumber'
        }
        if (attribute.isDeepInheritor('java.lang.String')) {
            return 'text'
        }
        if (attribute.isDeepInheritor('java.util.Date')) {
            def temporalAnnotation = attribute.getAnnotation('Temporal')
            if (temporalAnnotation != null) {
                def temporalType = temporalAnnotation.params['value']
                if (temporalType == 'TemporalType.DATE')
                    return 'date'
                if (temporalType == 'TemporalType.TIMESTAMP')
                    return 'dateTime'
                if (temporalType == 'TemporalType.TIME')
                    return 'time'
            }
        }
    }
    return null
}

def isToOneRelation(attribute) {
    return (attribute.isClass()
            && (attribute.hasAnnotation('ManyToOne')
                || attribute.hasAnnotation('OneToOne')))
}

def isFile(attribute) {
    return (isToOneRelation(attribute) && attribute.isDeepInheritor('com.haulmont.cuba.core.entity.FileDescriptor'))
}

return fields